<?php

/* Check if we need to quit !*/

/**
 * WP Core Web Vitals image conversion on-the-fly
 * 
 *  This script will convert images on the fly and cache them. If a WebP or Avid Image does not exist the request can be rewritten to this script. 
 *  It will generate the image on the fly, cache it and serve it.
 * 
 * - Each private function has the ability to stop the process (and set a Failure reason header).
 * - Each conversion method, on success will set a Success header mentioning the method
 */
class ImageConvert
{
    private $docroot;
    private $aAbsPath;
    private $origMimeType;

    // conversion methods to try (found in ./methods/cwv-[methodname].php)
    private $methods = ['imageavif', 'imagewebp', 'imagick', 'remote'];



    public function __construct()
    {
        // get some paths
        $this->getPaths();

        // check the request origin
        $this->checkRequestOrigin();

        // check if the original file exists
        $this->origMimeType = $this->checkOrigFileExists();

        // check if the WebP file exists
        $this->checkTargetFileExists();

        // check if an Error file exists, if it does do not attempt the same conversion over and over.
        $this->checkErrorFileExists();

        // create dir
        $this->createDir();

        // convert the image
        $this->convertImage();

        //maybe show the image
        $this->maybeShowFile();
    }


    /**
     * Create the directory if it does not exist
     */
    private function createDir()
    {
        // get dirname
        $sDir = dirname($this->aAbsPath['webp']);

        // create dir if it does not exist
        if (!file_exists($sDir)) {
            mkdir($sDir, 0777, true);
        }

        // check if the dir is writable
        if (!is_writable($sDir)) {
            header('Failed: Orig dir not writable');
        }
    }


    private function convertImage()
    {

        // load the converter base and interface classes
        require_once __DIR__ . '/methods/cwv_converter.php';
        require_once __DIR__ . '/methods/cwv_converter_interface.php';

        // loop through the methods
        foreach ($this->methods as $method) {

            $className = 'cwv_' . $method;
            $classFile = __DIR__ . '/methods/cwv_' . $method . '.php';

            if (file_exists($classFile)) {
                require_once $classFile;
                $class = new $className($this->aAbsPath, $this->origMimeType);
                $result = $class->convert();

                if (true === $result) {
                    return;
                }
            }
        }
    }


    /**
     *  get mime type of the image file
     */
    private function _imagegetMime($file)
    {
        try {
            if (is_callable('exif_imagetype')) {
                $imagetype = @exif_imagetype($file);
                $mime      = ($imagetype) ? image_type_to_mime_type($imagetype) : false;
            } elseif (function_exists('getimagesize')) {
                $imagesize = @getimagesize($file);
                $mime = (isset($imagesize['mime'])) ? $imagesize['mime'] : false;
            } else {
                $mime = false;
            }

            if (false !== $mime) {
                return $mime;
            }

            $magic = @file_get_contents($file, false, null, 0, 12);

            if (false === $magic) {
                return false;
            }

            $magic = bin2hex($magic);

            // detect WebP
            if (
                // RIFF.
                (0 === strpos($magic, '52494646')) &&
                // WEBP.
                (16 === strpos($magic, '57454250'))
            ) {
                $mime = 'image/webp';
            }

            // Detect AVIF
            if (strpos($magic, '6674797') !== false) {
                $mime = 'image/avif';
            }
        } catch (Exception $e) {
            $mime = false;
        }

        return $mime;
    }


    /**
     *  checks is image exists, no use in sending it to converter otherwise
     *  return mime type if it does exist
     *  @return string|bool
     */
    private function checkOrigFileExists()
    {
        $mime = $this->_imagegetMime($this->aAbsPath['orig']);
        if (false === $mime) {
            header('Failed: Orig file does not exist');
            $this->handleError();
        }


        return $mime;
    }

    /**
     *  checks if WebP OR AVIF file exists (if it does there is a config issue since we are executing the converter while the image exists)
     */
    private function checkTargetFileExists()
    {
        if (true === $this->_imagegetMime($this->aAbsPath['webp']) || true === $this->_imagegetMime($this->aAbsPath['avif'])) {
            header('Failed: Converted file does allready exist');
            $this->handleError();
        }
    }

    /**
     *  checks if there error file exists (if it does the image conversion failed, no need to retry)LALALA SETGGINGHSLALALA SETGGINGHS.
     */
    private function checkErrorFileExists()
    {
        if (file_exists($this->aAbsPath['error'])) {
            header('Failed: Errord file exist');
            $this->handleError();
        }
    }


    /**
     *    checks is the request is from the imagte converter.
     *    if it is we are in an infinite loop and there is an issue with the config.  
     **/
    private function checkRequestOrigin()
    {
        $bIsRequestFromConverter = ($_SERVER['HTTP_USER_AGENT'] == 'corewebvitals image converter') ? true : false;
        if ($bIsRequestFromConverter) {
            header('Failed: Request from converter');
            $this->handleError();
        }
    }

    /**
     * Show the image file if it exists, otherwise handle the error!
     */
    private function maybeShowFile()
    {

        if ($mime = $this->_imagegetMime($this->aAbsPath['avif'])) {

            header('Success:' . $mime);
            header('Content-type:' . $mime);
            readfile($this->aAbsPath['avif']);
            exit();
        }

        if ($mime = $this->_imagegetMime($this->aAbsPath['webp'])) {

            header('Success:' . $mime);
            header('Content-type:' . $mime);
            readfile($this->aAbsPath['webp']);
            exit();
        }
        header('Failed: Converted image is not valid');

        $this->handleError();
    }





    /**
     * get the paths for the original file and the webp file
     */
    private function getPaths()
    {

        $this->docroot = $_SERVER['DOCUMENT_ROOT'];

        $relpath = parse_url('https://example.com' . $_SERVER['REQUEST_URI'], PHP_URL_PATH);

        $replacepath = str_replace('/wp-content/', '/wp-content/cache/cwv_img/', $relpath);


        $this->aAbsPath = [
            'orig' => $this->docroot . $relpath,
            'webp' => $this->docroot . $replacepath . '.webp',
            'avif' => $this->docroot . $replacepath . '.avif',
            'error' => $this->docroot . $replacepath . '.webp.error'
        ];
    }

    /* saves an error reference and outputs a blanc pixel */
    public function handleError()
    {
        file_put_contents($this->aAbsPath['error'], '1');
        header('Image-conversion: live');
        header('Content-Type: image/png');
        echo base64_decode('iVBORw0KGgoAAAANSUhEUgAAAAEAAAABAQMAAAAl21bKAAAAA1BMVEUAAACnej3aAAAAAXRSTlMAQObYZgAAAApJREFUCNdjYAAAAAIAAeIhvDMAAAAASUVORK5CYII=');
        exit();
    }
}

new ImageConvert();
